@@ -17,13 +17,22 @@ from __future__ import annotations
 from abc import abstractmethod
 from dataclasses import dataclass
 
-from beartype.typing import Any, Generic, Sequence, TypeVar, cast
+from beartype.typing import Generic, Sequence, TypeVar, cast
 from typing_extensions import override
 
 from superlinked.framework.common.data_types import PythonTypes
 from superlinked.framework.common.interface.comparison_operand import ComparisonOperand
+from superlinked.framework.common.interface.comparison_operation_type import (
+    COMPARABLE_COMPARISON_OPERATION_TYPES,
+    CONTAINS_COMPARISON_OPERATION_TYPES,
+    EQUALITY_COMPARISON_OPERATION_TYPES,
+    ComparisonOperationType,
+)
 from superlinked.framework.common.schema.blob_information import BlobInformation
 from superlinked.framework.common.schema.exception import FieldException
+from superlinked.framework.common.schema.schema_field_descriptor import (
+    SchemaFieldDescriptor,
+)
 
 # Exclude from documentation.
 # A better approach would be to separate this file into atomic objects,
@@ -69,7 +78,7 @@ class SchemaObject:
         return f"{self.__class__.__name__}(schema_name={self._schema_name}, schema_fields=[{schema_fields}])"
 
     def _init_field(self: SchemaObjectT, field_descriptor: SchemaFieldDescriptor) -> SchemaField:
-        value = field_descriptor.type_(field_descriptor.name, self)
+        value = field_descriptor.type_(field_descriptor.name, self, field_descriptor.nullable)
         setattr(self, field_descriptor.name, value)
         return value
 
@@ -99,11 +108,12 @@ class SchemaField(ComparisonOperand, Generic[SFT]):
     to feed the vector embedding process.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT, type_: type[SFT]) -> None:
+    def __init__(self, name: str, schema_obj: SchemaObjectT, type_: type[SFT], nullable: bool) -> None:
         super().__init__(SchemaField)
         self.name = name
         self.schema_obj = schema_obj
         self.type_ = type_
+        self.nullable = nullable
 
     def parse(self, value: SFT) -> SFT:
         return value
@@ -117,6 +127,10 @@ class SchemaField(ComparisonOperand, Generic[SFT]):
             f"(name={self.name}, type={self.type_}, schema_object_name={self.schema_obj._schema_name})"
         )
 
+    @property
+    @abstractmethod
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]: ...
+
     @staticmethod
     def _built_in_equal(left_operand: ComparisonOperand[SchemaField], right_operand: object) -> bool:
         if isinstance(left_operand, SchemaField) and isinstance(right_operand, SchemaField):
@@ -128,6 +142,9 @@ class SchemaField(ComparisonOperand, Generic[SFT]):
         return not SchemaField._built_in_equal(left_operand, right_operand)
 
 
+SchemaFieldT = TypeVar("SchemaFieldT", bound=SchemaField)
+
+
 class String(SchemaField[str]):
     """
     Field of a schema that represents a string value.
@@ -135,14 +152,19 @@ class String(SchemaField[str]):
     e.g.: `TextEmbeddingSpace` expects a String field as an input.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, str)
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, str, nullable)
 
     def __add__(self, other: object) -> DescribedBlob:
         if not isinstance(other, Blob):
             raise TypeError(f"Operand must be of type {Blob.__name__}")
         return DescribedBlob(blob=other, description=self)
 
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return EQUALITY_COMPARISON_OPERATION_TYPES
+
 
 class Timestamp(SchemaField[int]):
     """
@@ -151,8 +173,13 @@ class Timestamp(SchemaField[int]):
     e.g.: `RecencySpace` expects a Timestamp field as an input.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, int)
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, int, nullable)
+
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return EQUALITY_COMPARISON_OPERATION_TYPES + COMPARABLE_COMPARISON_OPERATION_TYPES
 
 
 class Blob(SchemaField[BlobInformation]):
@@ -162,14 +189,19 @@ class Blob(SchemaField[BlobInformation]):
     e.g.: `ImageSpace` expects a blob field as an input.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, BlobInformation)
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, BlobInformation, nullable)
 
     def __add__(self, other: object) -> DescribedBlob:
         if not isinstance(other, String):
             raise TypeError(f"Operand must be of type {String.__name__}")
         return DescribedBlob(blob=self, description=other)
 
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return []
+
 
 NSFT = TypeVar("NSFT", float, int)
 
@@ -187,8 +219,13 @@ class Float(Number[float]):
     Field of a schema that represents a float.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, float)
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, float, nullable)
+
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return COMPARABLE_COMPARISON_OPERATION_TYPES
 
 
 class Integer(Number[int]):
@@ -196,8 +233,13 @@ class Integer(Number[int]):
     Field of a schema that represents an integer.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, int)
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, int, nullable)
+
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return EQUALITY_COMPARISON_OPERATION_TYPES + COMPARABLE_COMPARISON_OPERATION_TYPES
 
 
 class FloatList(SchemaField[list[float]]):
@@ -205,26 +247,36 @@ class FloatList(SchemaField[list[float]]):
     Field of a schema that represents a vector.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, list[float])
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, list[float], nullable)
 
     @override
     def parse(self, value: list[float]) -> list[float]:
         return value if isinstance(value, list) else [cast(float, value)]
 
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return []
+
 
 class StringList(SchemaField[list[str]]):
     """
     Field of a schema that represents a list of strings.
     """
 
-    def __init__(self, name: str, schema_obj: SchemaObjectT) -> None:
-        super().__init__(name, schema_obj, list[str])
+    def __init__(self, name: str, schema_obj: SchemaObjectT, nullable: bool) -> None:
+        super().__init__(name, schema_obj, list[str], nullable)
 
     @override
     def parse(self, value: list[str]) -> list[str]:
         return value if isinstance(value, list) else [cast(str, value)]
 
+    @property
+    @override
+    def supported_comparison_operation_types(self) -> Sequence[ComparisonOperationType]:
+        return CONTAINS_COMPARISON_OPERATION_TYPES
+
 
 @dataclass(frozen=True)
 class DescribedBlob:
@@ -234,10 +286,3 @@ class DescribedBlob:
 
 ConcreteSchemaField = String | Timestamp | Float | Integer | FloatList | StringList | Blob
 ConcreteSchemaFieldT = TypeVar("ConcreteSchemaFieldT", bound="ConcreteSchemaField")
-
-
-@dataclass
-class SchemaFieldDescriptor:
-    name: str
-    type_: type
-    type_args: tuple[Any, ...]
