@@ -19,13 +19,14 @@ from beartype.typing import Sequence
 from superlinked.framework.common.interface.comparison_operand import (
     ComparisonOperation,
 )
-from superlinked.framework.common.storage.field import Field
-from superlinked.framework.common.storage.field_data import VectorFieldData
+from superlinked.framework.common.storage.field.field import Field
+from superlinked.framework.common.storage.field.field_data import VectorFieldData
 
 
 @dataclass(frozen=True)
 class VDBKNNSearchParams:
     vector_field: VectorFieldData
     limit: int
+    fields_to_return: Sequence[Field]
     filters: Sequence[ComparisonOperation[Field]] | None
     radius: float | None
