@@ -14,10 +14,10 @@
 
 from dataclasses import dataclass
 
-from beartype.typing import Any, Sequence, TypeVar, cast
+from beartype.typing import Any, Mapping, Sequence, TypeVar, cast
 
 from superlinked.framework.common.dag.index_node import IndexNode
-from superlinked.framework.common.data_types import NodeDataTypes, PythonTypes
+from superlinked.framework.common.data_types import NodeDataTypes, PythonTypes, Vector
 from superlinked.framework.common.exception import InvalidSchemaException
 from superlinked.framework.common.interface.comparison_operand import (
     ComparisonOperation,
@@ -107,6 +107,7 @@ class StorageManager:
         index_node: IndexNode,
         schema: IdSchemaObject,
         knn_search_params: KNNSearchParams,
+        should_return_index_vector: bool = False,
         **params: Any,
     ) -> Sequence[SearchResultItem]:
         self._validate_knn_search_input(schema, knn_search_params.schema_fields_to_return)
@@ -117,6 +118,8 @@ class StorageManager:
         )
         schema_fields_by_fields = self._map_schema_fields_to_fields(knn_search_params.schema_fields_to_return)
         fields_to_return = list(schema_fields_by_fields.keys()) + list(self._entity_builder._admin_fields.header_fields)
+        if should_return_index_vector:
+            fields_to_return.append(self._entity_builder.compose_field(index_node.node_id, Vector))
         search_result: Sequence[ResultEntityData] = self._vdb_connector.knn_search(
             index_name,
             schema._schema_name,
@@ -131,14 +134,8 @@ class StorageManager:
         )
         schema_field_by_field_name = self._create_schema_field_by_field_name(schema_fields_by_fields)
         return [
-            SearchResultItem(
-                self._entity_builder._admin_fields.extract_header(result_entity_data.field_data),
-                [
-                    self._entity_builder.parse_schema_field_data(field_data, schema_field_by_field_name)
-                    for field_data in result_entity_data.field_data.values()
-                    if not self._entity_builder._admin_fields.is_admin_field(field_data)
-                ],
-                result_entity_data.score,
+            self._map_vdb_result_item_to_search_result_item(
+                result_entity_data, schema_field_by_field_name, index_node.node_id
             )
             for result_entity_data in search_result
         ]
@@ -177,17 +174,34 @@ class StorageManager:
             for filter_ in filters
         ] + [self._entity_builder._admin_fields.schema_id.field == schema._schema_name]
 
-    # TODO FAI-2737 to solve the parameters
-    def write_object_jsons(self, object_jsons: Sequence[tuple[IdSchemaObject, str, dict[str, Any]]]) -> None:
-        entities = [
-            self._entity_builder.compose_entity_data(
-                schema._schema_name,
-                id_,
-                [self._create_field_data_or_raise(data)],
-            )
-            for schema, id_, data in object_jsons
+    def _map_vdb_result_item_to_search_result_item(
+        self,
+        result_entity_data: ResultEntityData,
+        schema_field_by_field_name: Mapping[str, SchemaField],
+        index_node_id: str,
+    ) -> SearchResultItem:
+        parsed_schema_fields = [
+            self._entity_builder.parse_schema_field_data(field_data, schema_field_by_field_name)
+            for field_data in result_entity_data.field_data.values()
+            if not self._entity_builder._admin_fields.is_admin_field(field_data)
+            and field_data.name in schema_field_by_field_name
         ]
-        self._vdb_connector.write_entities(entities)
+        index_vector: Vector | None = None
+        if index_vector_field := next(
+            (
+                field_data
+                for field_name, field_data in result_entity_data.field_data.items()
+                if field_name == index_node_id
+            ),
+            None,
+        ):
+            index_vector = cast(Vector, index_vector_field.value)
+        return SearchResultItem(
+            self._entity_builder._admin_fields.extract_header(result_entity_data.field_data),
+            parsed_schema_fields,
+            result_entity_data.score,
+            index_vector,
+        )
 
     def write_parsed_schema_fields(self, parsed_schemas: Sequence[ParsedSchema]) -> None:
         entities_to_write = [
@@ -223,19 +237,6 @@ class StorageManager:
         field_data.extend(list(self._entity_builder.compose_field_data_from_node_data(node_id, node_data)))
         self._vdb_connector.write_entities([EntityData(entity_id, {fd.name: fd for fd in field_data})])
 
-    def read_object_jsons(self, schema: SchemaObject, object_ids: Sequence[str]) -> dict[str, dict[str, Any]]:
-        entities = [
-            self._entity_builder.compose_entity(
-                self._entity_builder.compose_entity_id(schema._schema_name, object_id),
-                [self._entity_builder._admin_fields.object_json.field],
-            )
-            for object_id in object_ids
-        ]
-
-        entities_data: Sequence[EntityData] = self._vdb_connector.read_entities(entities)
-
-        return {entity.id_.object_id: self._get_object_json_or_raise(entity) for entity in entities_data}
-
     def read_schema_field_values(self, object_id: str, schema_fields: Sequence[SchemaField]) -> ParsedSchema:
         schema = self._get_schema_of_schema_fields(schema_fields)
         entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
@@ -261,6 +262,25 @@ class StorageManager:
             raise InvalidSchemaException(f"The root schema of `schema_fields` must be an {IdSchemaObject.__name__}")
         return schema
 
+    def read_node_results(
+        self,
+        schemas_with_object_ids: Sequence[tuple[SchemaObject, str]],
+        node_id: str,
+        result_type: type[ResultTypeT],
+    ) -> list[ResultTypeT | None]:
+        entity_ids = [
+            self._entity_builder.compose_entity_id(schema._schema_name, object_id)
+            for schema, object_id in schemas_with_object_ids
+        ]
+        result_field = self._entity_builder.compose_field(node_id, result_type)
+        entities = [self._entity_builder.compose_entity(entity_id, [result_field]) for entity_id in entity_ids]
+        entity_data = self._vdb_connector.read_entities(entities)
+
+        def cast_value_if_not_none(field_data: FieldData | None) -> ResultTypeT | None:
+            return cast(ResultTypeT, field_data.value) if field_data else None
+
+        return [cast_value_if_not_none(ed.field_data.get(result_field.name)) for ed in entity_data]
+
     def read_node_result(
         self,
         schema: SchemaObject,
@@ -268,21 +288,9 @@ class StorageManager:
         node_id: str,
         result_type: type[ResultTypeT],
     ) -> ResultTypeT | None:
-        entity_id = self._entity_builder.compose_entity_id(schema._schema_name, object_id)
-        result_field = self._entity_builder.compose_field(node_id, result_type)
-        fields = [result_field]
-        entity = self._entity_builder.compose_entity(entity_id, fields)
-        entity_data = self._vdb_connector.read_entities([entity])[0]
-        result_value = next(
-            (
-                cast(ResultTypeT, field_data.value)
-                for field_data in entity_data.field_data.values()
-                if field_data.name == result_field.name
-            ),
-            None,
-        )
-        return result_value
+        return next(iter(self.read_node_results([(schema, object_id)], node_id, result_type)))
 
+    # TODO FAI-2737 to solve the parameters
     def read_node_data(
         self,
         schema: SchemaObject,
@@ -358,18 +366,3 @@ class StorageManager:
             )
             for key, type_ in node_data_descriptor.items()
         }
-
-    def _get_object_json_or_raise(self, entity_data: EntityData) -> dict[str, Any]:
-        object_json = self._entity_builder._admin_fields.extract_object_json_field_data(entity_data.field_data)
-        if not object_json:
-            raise ValueError(
-                f"Object JSON not found for schema_id={entity_data.id_.schema_id}, "
-                f"object_id={entity_data.id_.object_id}"
-            )
-        return object_json
-
-    def _create_field_data_or_raise(self, object_data: dict[str, Any]) -> FieldData:
-        field_data = self._entity_builder._admin_fields.create_object_json_field_data(object_data)
-        if not field_data:
-            raise ValueError(f"Failed to create object JSON field data from input: {object_data}")
-        return field_data
