@@ -14,7 +14,6 @@
 
 from beartype.typing import Sequence, TypeVar
 from furl import furl
-from pydantic.alias_generators import to_snake
 
 from superlinked.framework.dsl.executor.exception import DuplicateEndpointException
 from superlinked.framework.dsl.executor.rest.rest_configuration import (
@@ -22,7 +21,7 @@ from superlinked.framework.dsl.executor.rest.rest_configuration import (
     RestQuery,
 )
 from superlinked.framework.dsl.query.query_mixin import QueryMixin
-from superlinked.framework.dsl.query.result import Result
+from superlinked.framework.dsl.query.result import QueryResult
 from superlinked.framework.dsl.source.rest_source import RestSource
 
 REST = TypeVar("REST", RestSource, RestQuery)
@@ -37,19 +36,15 @@ class RestHandler:
         endpoint_config: RestEndpointConfiguration,
     ) -> None:
         self.__query_mixin = query_mixin
-        self.__path_to_source_map: dict[str, RestSource] = (
-            self.__create_path_to_resource_mapping(
-                sources,
-                endpoint_config.api_root_path,
-                endpoint_config.ingest_path_prefix,
-            )
+        self.__path_to_source_map: dict[str, RestSource] = self.__create_path_to_resource_mapping(
+            sources,
+            endpoint_config.api_root_path,
+            endpoint_config.ingest_path_prefix,
         )
-        self.__path_to_query_map: dict[str, RestQuery] = (
-            self.__create_path_to_resource_mapping(
-                queries,
-                endpoint_config.api_root_path,
-                endpoint_config.query_path_prefix,
-            )
+        self.__path_to_query_map: dict[str, RestQuery] = self.__create_path_to_resource_mapping(
+            queries,
+            endpoint_config.api_root_path,
+            endpoint_config.query_path_prefix,
         )
 
     @property
@@ -62,11 +57,13 @@ class RestHandler:
 
     def _ingest_handler(self, input_schema: dict, path: str) -> None:
         source = self.__path_to_source_map[path]
-        source._online_source.put(input_schema)
+        source.put([input_schema])
 
-    def _query_handler(self, query_object: dict, path: str) -> Result:
-        query = self.__path_to_query_map[path].query_obj
-        return self.__query_mixin.query(query, **query_object)
+    def _query_handler(self, query_descriptor: dict, path: str, include_metadata: bool = False) -> QueryResult:
+        query = self.__path_to_query_map[path].query_descriptor
+        if include_metadata:
+            query = query.include_metadata()
+        return self.__query_mixin.query(query, **query_descriptor)
 
     def __create_path_to_resource_mapping(
         self,
@@ -77,9 +74,7 @@ class RestHandler:
         path_to_resource = {}
         for resource in resources:
             path_string = str(
-                furl(path=api_root_path)
-                .path.add(path_prefix)
-                .add(to_snake(resource.path))  # type: ignore[arg-type]
+                furl(path=api_root_path).path.add(path_prefix).add(resource.path)  # type: ignore[arg-type]
             )
             if path_string in path_to_resource:
                 raise DuplicateEndpointException(
