@@ -14,7 +14,6 @@
 
 from __future__ import annotations
 
-from collections import defaultdict
 from collections.abc import Mapping
 
 import structlog
@@ -55,7 +54,6 @@ from superlinked.framework.dsl.query.param import (
 from superlinked.framework.dsl.query.predicate.binary_predicate import (
     EvaluatedBinaryPredicate,
     LooksLikePredicate,
-    SimilarPredicate,
 )
 from superlinked.framework.dsl.query.query_clause import (
     HardFilterClause,
@@ -97,16 +95,14 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         index: Index,
         schema: IdSchemaObject,
         clauses: Sequence[QueryClause] | None = None,
+        with_metadata: bool = False,
     ) -> None:
         self.__index = index
         self.__schema = schema
         self.__clauses: Sequence[QueryClause] = clauses if clauses else []
+        self.__with_metadata = with_metadata
         QueryDescriptorValidator.validate(self)
 
-    @property
-    def clauses(self) -> Sequence[QueryClause]:
-        return self.__clauses
-
     @property
     def index(self) -> Index:
         return self.__index
@@ -115,6 +111,14 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
     def schema(self) -> IdSchemaObject:
         return self.__schema
 
+    @property
+    def clauses(self) -> Sequence[QueryClause]:
+        return self.__clauses
+
+    @property
+    def with_metadata(self) -> bool:
+        return self.__with_metadata
+
     def space_weights(self, weight_by_space: Mapping[Space, NumericParamType]) -> QueryDescriptor:
         clauses = [SpaceWeightClause(self.__to_param(weight), space) for space, weight in weight_by_space.items()]
         altered_query_descriptor = self.__append_clauses(clauses)
@@ -122,7 +126,7 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
 
     def override_now(self, now: IntParamType) -> QueryDescriptor:
         clause = OverriddenNowClause(self.__to_param(now))
-        altered_query_descriptor = self.__append_clause(clause)
+        altered_query_descriptor = self.__append_clauses([clause])
         return altered_query_descriptor
 
     def similar(
@@ -152,15 +156,17 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         field_set = (
             space_field_set.space_field_set if isinstance(space_field_set, HasSpaceFieldSet) else space_field_set
         )
-        schema_field = field_set.get_field_for_schema(self.schema)
-        if not schema_field:
-            raise InvalidSchemaException(f"'find' ({type(self.schema)}) is not in similarity field's schema types.")
+        self.__validate_schema(field_set)
         value_param = self.__to_param(param)
         weight_param = self.__to_param(weight)
-        clause = SimilarFilterClause(value_param, weight_param, field_set, schema_field)
-        altered_query_descriptor = self.__append_clause(clause)
+        clause = SimilarFilterClause(value_param, weight_param, field_set)
+        altered_query_descriptor = self.__append_clauses([clause])
         return altered_query_descriptor
 
+    def __validate_schema(self, field_set: SpaceFieldSet) -> None:
+        if self.schema not in field_set.space._embedding_node_by_schema:
+            raise InvalidSchemaException(f"'find' ({type(self.schema)}) is not in similarity field's schema types.")
+
     def limit(self, limit: IntParamType | None) -> QueryDescriptor:
         """
         Set a limit to the number of results returned by the query.
@@ -173,7 +179,7 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         """
         param = self.__to_param(limit) if limit is not None else Param.init_default(constants.DEFAULT_LIMIT)
         clause = LimitClause(param)
-        altered_query_descriptor = self.__append_clause(clause)
+        altered_query_descriptor = self.__append_clauses([clause])
         return altered_query_descriptor
 
     def select(self, *fields: SchemaField | str | Param) -> QueryDescriptor:
@@ -197,7 +203,7 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         """
         param = self.__handle_select_param(list(fields))
         clause = SelectClause(param)
-        altered_query_descriptor = self.__append_clause(clause)
+        altered_query_descriptor = self.__append_clauses([clause])
         return altered_query_descriptor
 
     def __handle_select_param(self, fields: Sequence[SchemaField | str | Param]) -> Param | Evaluated[Param]:
@@ -264,7 +270,7 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         """
         param = self.__to_param(radius) if radius is not None else Param.init_default()
         clause = RadiusClause(param)
-        return self.__append_clause(clause)
+        return self.__append_clauses([clause])
 
     def with_vector(
         self,
@@ -291,7 +297,7 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         value_param = self.__to_param(id_param)
         weight_param = self.__to_param(weight)
         clause = LooksLikeFilterClause(value_param, weight_param, schema_obj.id)
-        return self.__append_clause(clause)
+        return self.__append_clauses([clause])
 
     def filter(self, comparison_operation: ComparisonOperation[SchemaField] | _Or) -> QueryDescriptor:
         """
@@ -322,7 +328,9 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
             comparison_operation.operations if isinstance(comparison_operation, _Or) else [comparison_operation]
         )
         for operation in comparison_operations:
+            QueryFilterValidator.validate_operation_is_supported(operation)
             QueryFilterValidator.validate_operation_operand_type(operation, allow_param=True)
+            QueryFilterValidator.validate_operation_field_is_part_of_schema(operation, self.__schema)
         clauses = [
             HardFilterClause(
                 self.__to_param(operation._other),
@@ -336,6 +344,17 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         self.__warn_if_nlq_is_used_without_recommended_param_descriptions(altered_query_descriptor)
         return altered_query_descriptor
 
+    def include_metadata(self) -> QueryDescriptor:
+        """
+        Make per-item metadata to be returned in the query results.
+
+        Current metadata includes space-wise partial scores.
+
+        Returns:
+            Self: The query object itself.
+        """
+        return QueryDescriptor(self.index, self.schema, self.clauses, with_metadata=True)
+
     def nlq_suggestions(self, feedback: str | None = None) -> QuerySuggestionsModel:
         """
         Get suggestions for improving the natural language query parameters.
@@ -388,7 +407,7 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
 
     def get_selected_fields(self) -> Sequence[SchemaField]:
         field_names = self.get_mandatory_clause_by_type(SelectClause).evaluate()
-        return self.schema._get_fields_by_names(field_names)
+        return self._get_selected_fields_by_names(field_names)
 
     def get_hard_filters(self) -> list[ComparisonOperation[SchemaField]]:
         return [
@@ -407,16 +426,9 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         looks_like_filter = looks_like_clause.evaluate() if looks_like_clause is not None else None
         return looks_like_filter
 
-    def get_similar_filters(
-        self,
-    ) -> dict[Space, list[EvaluatedBinaryPredicate[SimilarPredicate]]]:
-        similar_filters_by_space = defaultdict(list)
-        for clause in self.get_clauses_by_type(SimilarFilterClause):
-            space_and_similar_filter = clause.evaluate()
-            if space_and_similar_filter is not None:
-                space, similar = space_and_similar_filter
-                similar_filters_by_space[space].append(similar)
-        return dict(similar_filters_by_space)
+    def get_similar_filters_spaces(self) -> list[Space]:
+        evaluation_results = [clause.evaluate() for clause in self.get_clauses_by_type(SimilarFilterClause)]
+        return [result[0] for result in evaluation_results if result is not None]
 
     def get_context_time(self, default: int | Any) -> int:
         if (overridden_now_clause := self.get_clause_by_type(OverriddenNowClause)) is not None:
@@ -447,11 +459,11 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         unset_space_by_param_name = {
             clause.value_param.name: clause.space
             for clause in self.get_clauses_by_type(SpaceWeightClause)
-            if not isinstance(clause.value_param, Evaluated)
+            if not isinstance(clause.value_param, Evaluated) and clause.value_param.default is None
         }
         if self.get_looks_like_filter() is not None:
             return {param_name: constants.DEFAULT_WEIGHT for param_name in unset_space_by_param_name.keys()}
-        similar_filter_spaces = self.get_similar_filters().keys()
+        similar_filter_spaces = self.get_similar_filters_spaces()
         return {
             param_name: (
                 constants.DEFAULT_WEIGHT if space in similar_filter_spaces else constants.DEFAULT_NOT_AFFECTING_WEIGHT
@@ -477,21 +489,19 @@ class QueryDescriptor:  # pylint: disable=too-many-public-methods
         return [clause for clause in self.clauses if isinstance(clause, clause_type)]
 
     def replace_clauses(self, clauses: Sequence[QueryClause]) -> QueryDescriptor:
-        return QueryDescriptor(self.index, self.schema, clauses)
-
-    def calculate_value_by_param_name(self) -> dict[str, Any]:
-        value_by_param_name = {clause.value_param_name: clause.get_value() for clause in self.clauses}
-        weight_by_param_name = {clause.weight_param_name: clause.get_weight() for clause in self.get_weighted_clauses()}
-        return {**value_by_param_name, **weight_by_param_name}
+        return QueryDescriptor(self.index, self.schema, clauses, self.with_metadata)
 
     def get_weighted_clauses(self) -> list[WeightedQueryClause]:
         return [clause for clause in self.clauses if isinstance(clause, WeightedQueryClause)]
 
-    def __append_clause(self, clause: QueryClause) -> QueryDescriptor:
-        return self.__append_clauses([clause])
-
     def __append_clauses(self, clauses: Sequence[QueryClause]) -> QueryDescriptor:
-        return QueryDescriptor(self.index, self.schema, list(self.clauses) + list(clauses))
+        return QueryDescriptor(self.index, self.schema, list(self.clauses) + list(clauses), self.with_metadata)
+
+    def _get_selected_fields_by_names(self, field_names: Sequence[str]) -> list[SchemaField]:
+        """We filter out the id field as it is always returned."""
+        id_field_name = self.schema.id.name
+        filtered_field_names = [name for name in field_names if name != id_field_name]
+        return self.schema._get_fields_by_names(filtered_field_names)
 
     @classmethod
     def __to_param(cls, param_input: Any) -> Param | Evaluated[Param]:
@@ -611,7 +621,9 @@ class QueryDescriptorValidator:
         if clause is None:
             return
         field_names = clause.get_value()
-        query_descriptor.schema._get_fields_by_names(field_names)  # this also validates
+        if query_descriptor.schema.id.name in field_names:
+            logger.info("The id field is automatically included - no need to specify it in the select clause.")
+        query_descriptor._get_selected_fields_by_names(field_names)  # this also validates
 
     @staticmethod
     def __validate_overridden_now_clause(query_descriptor: QueryDescriptor) -> None:
