@@ -35,7 +35,7 @@ from superlinked.framework.common.space.config.embedding.categorical_similarity_
     CategoricalSimilarityEmbeddingConfig,
 )
 from superlinked.framework.common.space.config.normalization.normalization_config import (
-    L2NormConfig,
+    CategoricalNormConfig,
 )
 from superlinked.framework.common.space.config.transformation_config import (
     TransformationConfig,
@@ -85,7 +85,7 @@ class CategoricalSimilaritySpace(Space[Vector, list[str]], HasSpaceFieldSet):
 
     def __init__(
         self,
-        category_input: StringList | String | list[String | StringList],
+        category_input: String | StringList | None | list[String | StringList | None],
         categories: list[str],
         negative_filter: float = 0.0,
         uncategorized_as_category: bool = True,
@@ -113,16 +113,14 @@ class CategoricalSimilaritySpace(Space[Vector, list[str]], HasSpaceFieldSet):
             InvalidSchemaException: If a schema object does not have a corresponding node in the similarity space,
             indicating a configuration or implementation error.
         """
-        TypeValidator.validate_list_item_type(categories, str, "categories")
-        # TODO FAI-2843 this type ignore is not needed but linting is flaky in CI
+        non_none_category_input: list[String | StringList] = self._fields_to_non_none_sequence(category_input)
+        # This type ignore is not needed but linting is flaky in CI.
         super().__init__(
-            category_input,
+            non_none_category_input,
             String | StringList,  # type: ignore[misc] # interface supports only one type
         )
-        self.__category = SpaceFieldSet[list[str]](
-            self,
-            set(category_input if isinstance(category_input, list) else [category_input]),
-        )
+        TypeValidator.validate_list_item_type(categories, str, "categories")
+        self.__category = SpaceFieldSet[list[str]](self, set(non_none_category_input))
         self._embedding_config = CategoricalSimilarityEmbeddingConfig(
             list[str],
             categories=categories,
@@ -137,7 +135,7 @@ class CategoricalSimilaritySpace(Space[Vector, list[str]], HasSpaceFieldSet):
                 transformation_config=self.transformation_config,
                 fields_for_identification=self.__category.fields,
             )
-            for single_category in (category_input if isinstance(category_input, list) else [category_input])
+            for single_category in non_none_category_input
         }
         self.__schema_node_map: dict[SchemaObject, EmbeddingNode[Vector, list[str]]] = {
             schema_field.schema_obj: node for schema_field, node in unchecked_category_node_map.items()
@@ -151,10 +149,11 @@ class CategoricalSimilaritySpace(Space[Vector, list[str]], HasSpaceFieldSet):
         return self.__schema_node_map
 
     def _init_transformation_config(
-        self, embedding_config: CategoricalSimilarityEmbeddingConfig
+        self,
+        embedding_config: CategoricalSimilarityEmbeddingConfig,
     ) -> TransformationConfig[Vector, list[str]]:
         aggregation_config = VectorAggregationConfig(Vector)
-        normalization_config = L2NormConfig()
+        normalization_config = CategoricalNormConfig(len(self._embedding_config.categories))
         return TransformationConfig(normalization_config, aggregation_config, embedding_config)
 
     @override
