@@ -14,64 +14,69 @@
 
 from __future__ import annotations
 
-from beartype.typing import cast
+from beartype.typing import Sequence
 from typing_extensions import override
 
 from superlinked.framework.common.dag.categorical_similarity_node import (
     CategoricalSimilarityNode,
 )
 from superlinked.framework.common.dag.context import ExecutionContext
-from superlinked.framework.common.dag.node import Node
 from superlinked.framework.common.data_types import Vector
 from superlinked.framework.common.interface.has_length import HasLength
 from superlinked.framework.common.parser.parsed_schema import ParsedSchema
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
+from superlinked.framework.common.transform.transform import Step
+from superlinked.framework.common.transform.transformation_factory import (
+    TransformationFactory,
+)
 from superlinked.framework.online.dag.evaluation_result import EvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
-from superlinked.framework.online.dag.parent_validator import ParentValidationType
 
 
-class OnlineCategoricalSimilarityNode(
-    OnlineNode[CategoricalSimilarityNode, Vector], HasLength
-):
+class OnlineCategoricalSimilarityNode(OnlineNode[CategoricalSimilarityNode, Vector], HasLength):
     def __init__(
         self,
         node: CategoricalSimilarityNode,
         parents: list[OnlineNode],
         storage_manager: StorageManager,
     ) -> None:
-        super().__init__(
-            node,
-            parents,
-            storage_manager,
-            ParentValidationType.LESS_THAN_TWO_PARENTS,
+        super().__init__(node, parents, storage_manager)
+        self._embedding_transformation = TransformationFactory.create_embedding_transformation(
+            self.node.transformation_config
         )
 
     @property
+    @override
     def length(self) -> int:
         return self.node.length
 
+    @property
+    def embedding_transformation(self) -> Step[list[str], Vector]:
+        return self._embedding_transformation
+
     @override
     def evaluate_self(
         self,
-        parsed_schemas: list[ParsedSchema],
+        parsed_schemas: Sequence[ParsedSchema],
         context: ExecutionContext,
-    ) -> list[EvaluationResult[Vector]]:
-        return [self.evaluate_self_single(schema, context) for schema in parsed_schemas]
-
-    def evaluate_self_single(
-        self,
-        parsed_schema: ParsedSchema,
-        context: ExecutionContext,
-    ) -> EvaluationResult[Vector]:
-        if context.should_load_default_node_input:
-            result = self.node.embedding.default_vector
-        elif len(self.parents) == 0:
-            result = self.load_stored_result_or_raise_exception(parsed_schema)
+    ) -> list[EvaluationResult[Vector] | None]:
+        if len(self.parents) == 0:
+            results = self.load_stored_results_with_default(
+                [(parsed_schema.schema, parsed_schema.id_) for parsed_schema in parsed_schemas],
+                Vector.init_zero_vector(self.node.length),
+            )
         else:
-            input_: EvaluationResult[list[str]] = cast(
-                OnlineNode[Node[list[str]], list[str]],
-                self.parents[0],
-            ).evaluate_next_single(parsed_schema, context)
-            result = self.node.embedding.embed(input_.main.value, context)
-        return EvaluationResult(self._get_single_evaluation_result(result))
+            parent_results = self.evaluate_parent(self.parents[0], parsed_schemas, context)
+            results = [self._evaluate_parent_result(parent_result, context) for parent_result in parent_results]
+        return [self._wrap_in_evaluation_result(result) for result in results]
+
+    def _evaluate_parent_result(
+        self,
+        parent_result: EvaluationResult | None,
+        context: ExecutionContext,
+    ) -> Vector:
+        if parent_result is None:
+            return Vector.init_zero_vector(self.node.length)
+        input_ = parent_result.main.value
+        categories = input_ if isinstance(input_, list) else [input_]
+        return self.embedding_transformation.transform(categories, context)
