@@ -22,10 +22,13 @@ from superlinked.framework.common.dag.recency_node import RecencyNode
 from superlinked.framework.common.data_types import Vector
 from superlinked.framework.common.interface.has_length import HasLength
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
+from superlinked.framework.common.transform.transform import Step
+from superlinked.framework.common.transform.transformation_factory import (
+    TransformationFactory,
+)
 from superlinked.framework.online.dag.default_online_node import DefaultOnlineNode
 from superlinked.framework.online.dag.evaluation_result import SingleEvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
-from superlinked.framework.online.dag.parent_validator import ParentValidationType
 
 DAY_IN_SECONDS: int = 24 * 60 * 60
 
@@ -37,27 +40,27 @@ class OnlineRecencyNode(DefaultOnlineNode[RecencyNode, Vector], HasLength):
         parents: list[OnlineNode],
         storage_manager: StorageManager,
     ) -> None:
-        super().__init__(
-            node,
-            parents,
-            storage_manager,
-            ParentValidationType.EXACTLY_ONE_PARENT,
+        super().__init__(node, parents, storage_manager)
+        self._embedding_transformation = TransformationFactory.create_embedding_transformation(
+            self.node.transformation_config
         )
 
     @property
+    @override
     def length(self) -> int:
-        return self.node.embedding.length
+        return self.node.length
+
+    @property
+    def embedding_transformation(self) -> Step[int, Vector]:
+        return self._embedding_transformation
 
     @override
     def _evaluate_singles(
         self,
-        parent_results: list[dict[OnlineNode, SingleEvaluationResult]],
+        parent_results: Sequence[dict[OnlineNode, SingleEvaluationResult]],
         context: ExecutionContext,
     ) -> Sequence[Vector | None]:
-        return [
-            self._evaluate_single(parent_result, context)
-            for parent_result in parent_results
-        ]
+        return [self._evaluate_single(parent_result, context) for parent_result in parent_results]
 
     def _evaluate_single(
         self,
@@ -66,8 +69,5 @@ class OnlineRecencyNode(DefaultOnlineNode[RecencyNode, Vector], HasLength):
     ) -> Vector | None:
         if len(parent_results.items()) != 1:
             return None
-        input_ = list(parent_results.values())[0]
-        return self._calc_recency(input_.value, context)
-
-    def _calc_recency(self, created_at: int, context: ExecutionContext) -> Vector:
-        return self.node.embedding.calc_recency_vector(created_at, context)
+        input_: int = list(parent_results.values())[0].value
+        return self.embedding_transformation.transform(input_, context)
