@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from beartype.typing import cast
+from beartype.typing import Sequence, cast
 
 from superlinked.framework.common.dag.context import ExecutionContext
 from superlinked.framework.common.dag.exception import (
@@ -58,31 +58,25 @@ class OnlineSchemaDag:
 
     def evaluate(
         self,
-        parsed_schemas: list[ParsedSchema],
+        parsed_schemas: Sequence[ParsedSchema],
         context: ExecutionContext,
-    ) -> list[EvaluationResult[Vector]]:
+    ) -> list[EvaluationResult[Vector] | None]:
         return self.leaf_node.evaluate_next(parsed_schemas, context)
 
     def __validate(self, schema: SchemaObject, nodes: list[OnlineNode]) -> None:
-        class_name = self.__class__.__name__
+        class_name = type(self).__name__
         leaf_nodes = [node for node in nodes if len(node.children) == 0]
         if len(leaf_nodes) != 1:
-            raise LeafNodeCountException(
-                f"{class_name} must have exactly one leaf Node."
-            )
+            raise LeafNodeCountException(f"{class_name} must have exactly one leaf Node.")
         if not isinstance(leaf_nodes[0], OnlineIndexNode):
-            raise LeafNodeTypeException(
-                f"{class_name} must have a OnlineIndexNode leaf Node."
-            )
+            raise LeafNodeTypeException(f"{class_name} must have a OnlineIndexNode leaf Node.")
         root_node_schemas = set(
             cast(SchemaFieldNode, node.node).schema_field.schema_obj
             for node in nodes
             if isinstance(node, OnlineSchemaFieldNode)
         )
         if len(root_node_schemas) != 1:
-            raise MultipleRootSchemaException(
-                f"{class_name}'s nodes must have the same root schema."
-            )
+            raise MultipleRootSchemaException(f"{class_name}'s nodes must have the same root schema.")
         if schema not in root_node_schemas:
             raise SchemaMismatchException(
                 (
