@@ -23,15 +23,14 @@ class ParentValidationType(Enum):
     LESS_THAN_TWO_PARENTS = (lambda len: len < 2, "less than 2 parents")
     AT_LEAST_ONE_PARENT = (lambda len: len >= 1, "at least 1 parent")
     EXACTLY_ONE_PARENT = (lambda len: len == 1, "exactly 1 parent")
+    ONE_OR_TWO_PARENTS = (lambda len: len in [1, 2], "1 or 2 parents")
     NO_PARENTS = (lambda len: len == 0, "no parents")
     NO_VALIDATION = (lambda len: True, "no validation")
 
     validator: Callable[[int], bool]
     description: str
 
-    def __new__(
-        cls, validator: Callable[[int], bool], description: str
-    ) -> ParentValidationType:
+    def __new__(cls, validator: Callable[[int], bool], description: str) -> ParentValidationType:
         obj = object.__new__(cls)
         obj.validator = validator
         obj.description = description
